import json
import shutil
from pathlib import Path

import hydra
import requests
from loguru import logger
from omegaconf import DictConfig
from yt_dlp import YoutubeDL


@hydra.main(version_base=None, config_path="conf", config_name="config")
def main(cfg: DictConfig) -> None:
    output_path = Path(cfg.scraper.output_folder)
    output_path.mkdir(exist_ok=True, parents=True)
    tmp_path = output_path / 'tmp'

    urls_path = Path(cfg.search.output_folder) / cfg.search.result_file
    try:
        with open(urls_path) as f:
            urls = list(dict.fromkeys(s.strip() for s in f.readlines()))
    except FileNotFoundError:
        logger.error(f'Urls file not found: {urls_path}')
        return

    audio_format: str = cfg.scraper.audio_format
    download_lvl: str = cfg.scraper.lvl

    ydl_opts = {
        'chapter': True,
        'format': 'm4a/bestaudio/best',
        'postprocessors': [{
            'key': 'FFmpegExtractAudio',
            'preferredcodec': f'{audio_format}',
            # 'preferredquality': '10',
        }],
        'paths': dict(home=str(tmp_path)),
        'outtmpl': 'audio.%(ext)s'
    }
    with YoutubeDL(ydl_opts) as ydl:
        for url in urls:
            if tmp_path.exists():
                shutil.rmtree(tmp_path)
            tmp_path.mkdir()

            video_info = ydl.extract_info(url, download=False)
            video_id = video_info['id']
            path = output_path / video_id
            if path.exists():
                logger.warning(f'Audio {video_info.get("id")} downloaded. Skipping.')
                continue

            chapters = video_info['chapters']
            auto_subs = True
            if video_info.get('subtitles') is None or video_info.get('subtitles').get('ru') is None:
                subs_data = video_info['automatic_captions']['ru-orig']
            else:
                subs_data = video_info['subtitles']['ru']
                auto_subs = False

            try:
                subs_link = [s for s in subs_data if s['ext'] == 'json3'][0]['url']
            except IndexError:
                logger.error(f'Incorrect subs type: {video_id}')
                continue

            summary = dict(
                auto_subs=auto_subs,
                id=video_info['id'],
                title=video_info['title'],
                description=video_info['description'],
                url=video_info['webpage_url'],
                categories=video_info['categories'],
                tags=video_info['tags'],
                date=video_info['upload_date'],
                duration=video_info['duration_string'],
                chapters=chapters,
            )
            json.dump(summary, open(tmp_path / 'info.json', 'w', encoding='utf8'), ensure_ascii=False, indent=2)

            subs = requests.get(subs_link).json()
            json.dump(subs, open(tmp_path / 'subs.json', 'w', encoding='utf8'), ensure_ascii=False)

            if download_lvl != 'text':
                ydl.download(url)

            shutil.move(tmp_path, path)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['src', 'src.conf']

package_data = \
{'': ['*']}

install_requires = \
['hydra-core>=1.3.2,<2.0.0',
 'loguru>=0.7.0,<0.8.0',
 'requests>=2.28.2,<3.0.0',
 'yt-dlp>=2023.3.4,<2024.0.0']

entry_points = \
{'console_scripts': ['search = src.search:main',
                     'subscrape = src.subscraper:main']}

setup_kwargs = {
    'name': 'subsscraper',
    'version': '0.1.4',
    'description': '',
    'long_description': 'SubsScraper\n---------\n\nRun command for generate urls\n```commandline\nsearch\nsearch search.cnt=2 # for 2 urls\nsearch \'search.query="лекции таймкоды"\' search.cnt=2 # override cyrillic (more quotes)\n```\n\nRun command for subscrape urls\n```commandline\nsubscrape\nsearch scraper.audio_format=mp3 # convert to audio format\nsearch scraper.lvl=text # download only text\n```\n\nDefault configs\n```commandline\nsearch:\n  output_folder: data\n  result_file: urls.txt\n  cnt: 10\n  query: машинное обучение лекции таймкоды\n\nscraper:\n  output_folder: ${search.output_folder}/dataset\n  audio_format: m4a\n  lvl: audio # text, audio, video\n```\n\n',
    'author': 'tupiznak',
    'author_email': 'akej-vonavi@mail.ru',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from pathlib import Path
from typing import Any

import hydra
from loguru import logger
from omegaconf import DictConfig
from yt_dlp import YoutubeDL


def process(path: str, info: dict[str, Any]) -> None:
    if info.get('chapters') is None:
        return
    # if info.get('subtitles') is None or info.get('subtitles').get('ru') is None:
    #     return
    if info.get('automatic_captions').get('ru-orig') is None:
        return
    url = info.get('original_url')
    logger.info(f"Got url: {url}")
    with open(path, 'a') as f:
        f.write(f'{url}\r\n')


class Wrap(YoutubeDL):
    path: str

    def process_ie_result(self, ie_result, download=True, extra_info=None):
        if ie_result.get('_type', None) is None:
            process(self.path, ie_result)
        return super().process_ie_result(ie_result, download, extra_info)


@hydra.main(version_base=None, config_path="conf", config_name="config")
def main(cfg: DictConfig) -> None:
    ydl_opts = {
        'writesubtitles': True,
        'subtitleslangs': ['ru'],
        'chapter': True,
        'format': 'm4a/bestaudio/best/ru',
        'default_search': f'ytsearch{cfg.search.cnt}:{cfg.search.query}',
    }
    with Wrap(ydl_opts) as ydl:
        ydl.path = Path(cfg.search.output_folder)
        ydl.path.mkdir(exist_ok=True)
        ydl.path /= cfg.search.result_file
        ydl.extract_info('', download=False)


if __name__ == '__main__':
    main()
